@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019 Mathieu Parent <math.parent@gmail.com>
+# Copyright (C) 2019-2020 Mathieu Parent <math.parent@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -54,6 +54,9 @@ class LabelsMixin(object):
                                     self._name, current_label_name, target_label_param_name,
                                          None, target_label_param_value)
                         continue
+                    if target_label_param_name == 'description' \
+                       and current_label_param_value is None:
+                        current_label_param_value = ''
                     if current_label_param_value != target_label_param_value:
                         if dry_run:
                             logger.info('[%s] NOT Changing label %s %s: %s -> %s (dry-run)',
