@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019 Mathieu Parent <math.parent@gmail.com>
+# Copyright (C) 2019-2020 Mathieu Parent <math.parent@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -44,7 +44,7 @@ class VariablesMixin(object):
                     try:
                         current_variable_param_value = getattr(current_variable, target_variable_param_name)
                     except AttributeError:
-                        logger.info('[%s] NOT Changing variables %s %s: %s -> %s (current value is not available)',
+                        logger.info('[%s] NOT Changing variable %s %s: %s -> %s (current value is not available)',
                                     self._name, current_variable_key, target_variable_param_name,
                                     None, target_variable_param_value)
                         continue
