@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019 Mathieu Parent <math.parent@gmail.com>
+# Copyright (C) 2019-2020 Mathieu Parent <math.parent@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -16,9 +16,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gitlab.exceptions import GitlabGetError
-
-import gitlabracadabra.manager
+from gitlabracadabra.mixins.groups import GroupsMixin
 from gitlabracadabra.mixins.labels import LabelsMixin
 from gitlabracadabra.mixins.members import MembersMixin
 from gitlabracadabra.mixins.milestones import MilestonesMixin
@@ -26,7 +24,8 @@ from gitlabracadabra.mixins.variables import VariablesMixin
 from gitlabracadabra.objects.object import GitLabracadabraObject
 
 
-class GitLabracadabraGroup(GitLabracadabraObject, LabelsMixin, MembersMixin, MilestonesMixin, VariablesMixin):
+class GitLabracadabraGroup(GitLabracadabraObject, GroupsMixin, LabelsMixin, MembersMixin, MilestonesMixin,
+                           VariablesMixin):
     EXAMPLE_YAML_HEADER = 'mygroup/:\n'
     DOC = [
         '# Group lifecycle',
@@ -61,6 +60,8 @@ class GitLabracadabraGroup(GitLabracadabraObject, LabelsMixin, MembersMixin, Mil
         '# Members',
         'members',
         'unknown_members',
+        'groups',
+        'unknown_groups',
 
         '# Labels',
         'labels',
@@ -210,6 +211,26 @@ class GitLabracadabraGroup(GitLabracadabraObject, LabelsMixin, MembersMixin, Mil
                 "description": "What to do with unknown members (`warn` by default).",
                 "enum": ["warn", "delete", "remove", "ignore", "skip"],
             },
+            # From https://docs.gitlab.com/ee/api/groups.html#create-a-link-to-share-a-group-with-another-group
+            # and https://docs.gitlab.com/ee/api/groups.html#delete-link-sharing-group-with-another-group
+            # FIXME expires_at not supported
+            "groups": {
+                "type": "object",
+                "description": "Groups",
+                "additionalProperties": {
+                    "type": "string",
+                    "description": "The permissions level to grant the group.",
+                    "enum": ["guest", "reporter", "developer", "maintainer"],
+                },
+                "_example": ("\n"
+                             "    group/foo: guest\n"
+                             "    group/bar: reporter # one of guest, reporter, developer, maintainer\n"),
+            },
+            "unknown_groups": {  # GitLabracadabra
+                "type": "string",
+                "description": "What to do with unknown groups (`warn` by default).",
+                "enum": ["warn", "delete", "remove", "ignore", "skip"],
+            },
             # From https://docs.gitlab.com/ee/api/group_labels.html#create-a-new-group-label
             "labels": {
                 "type": "array",
@@ -336,52 +357,6 @@ class GitLabracadabraGroup(GitLabracadabraObject, LabelsMixin, MembersMixin, Mil
         "additionalProperties": False,
     }
 
-    IGNORED_PARAMS = ['unknown_labels', 'unknown_members', 'unknown_milestones', 'unknown_variables']
+    IGNORED_PARAMS = ['unknown_groups', 'unknown_labels', 'unknown_members', 'unknown_milestones', 'unknown_variables']
 
     CREATE_KEY = 'name'
-
-    """"Groups mapping
-
-    indexed by id and full path.
-    """
-    _GROUPS_PATH2ID = {}
-    _GROUPS_ID2PATH = {}
-
-    """"Map group id and full path
-    """
-    @classmethod
-    def map_group(cls, group_id, group_full_path):
-        cls._GROUPS_ID2PATH[group_id] = group_full_path
-        cls._GROUPS_PATH2ID[group_full_path] = group_id
-
-    """"Get group full path from id
-    """
-    @classmethod
-    def get_full_path_from_id(cls, group_id):
-        if group_id not in cls._GROUPS_ID2PATH:
-            try:
-                obj_manager = gitlabracadabra.manager.get_manager().groups
-                group = obj_manager.get(group_id)
-                cls._GROUPS_ID2PATH[group.id] = group.full_path
-                cls._GROUPS_PATH2ID[group.full_path] = group.id
-            except GitlabGetError as e:
-                if e.response_code != 404:
-                    pass
-                cls._GROUPS_ID2PATH[group_id] = None
-        return cls._GROUPS_ID2PATH[group_id]
-
-    """"Get group id from full path
-    """
-    @classmethod
-    def get_id_from_full_path(cls, group_full_path):
-        if group_full_path not in cls._GROUPS_PATH2ID:
-            try:
-                obj_manager = gitlabracadabra.manager.get_manager().groups
-                group = obj_manager.get(group_full_path)
-                cls._GROUPS_ID2PATH[group.id] = group.full_path
-                cls._GROUPS_PATH2ID[group.full_path] = group.id
-            except GitlabGetError as e:
-                if e.response_code != 404:
-                    pass
-                cls._GROUPS_PATH2ID[group_full_path] = None
-        return cls._GROUPS_PATH2ID[group_full_path]
