@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019 Mathieu Parent <math.parent@gmail.com>
+# Copyright (C) 2019-2020 Mathieu Parent <math.parent@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -17,17 +17,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
-from copy import deepcopy
 
 from gitlab.exceptions import GitlabCreateError, GitlabListError
 
+from gitlabracadabra.mixins.groups import GroupsMixin
 from gitlabracadabra.mixins.labels import LabelsMixin
 from gitlabracadabra.mixins.members import MembersMixin
 from gitlabracadabra.mixins.milestones import MilestonesMixin
 from gitlabracadabra.mixins.mirrors import MirrorsMixin
+from gitlabracadabra.mixins.pipeline_schedules import PipelineSchedulesMixin
 from gitlabracadabra.mixins.variables import VariablesMixin
 from gitlabracadabra.mixins.webhooks import WebhooksMixin
-from gitlabracadabra.objects.group import GitLabracadabraGroup
 from gitlabracadabra.objects.object import GitLabracadabraObject
 from gitlabracadabra.utils import access_level
 
@@ -35,8 +35,8 @@ from gitlabracadabra.utils import access_level
 logger = logging.getLogger(__name__)
 
 
-class GitLabracadabraProject(GitLabracadabraObject, LabelsMixin, MembersMixin, MilestonesMixin, MirrorsMixin,
-                             VariablesMixin, WebhooksMixin):
+class GitLabracadabraProject(GitLabracadabraObject, GroupsMixin, LabelsMixin, MembersMixin, MilestonesMixin,
+                             MirrorsMixin, PipelineSchedulesMixin, VariablesMixin, WebhooksMixin):
     EXAMPLE_YAML_HEADER = 'mygroup/myproject:\n'
     DOC = [
         '# Project lifecycle',
@@ -149,6 +149,11 @@ class GitLabracadabraProject(GitLabracadabraObject, LabelsMixin, MembersMixin, M
         'milestones',
         'unknown_milestones',
 
+        '# Pipeline schedules',
+        'pipeline_schedules',
+        'unknown_pipeline_schedules',
+        'unknown_pipeline_schedule_variables',
+
         '# Deprecated',
         'issues_enabled',
         'merge_requests_enabled',
@@ -752,6 +757,98 @@ class GitLabracadabraProject(GitLabracadabraObject, LabelsMixin, MembersMixin, M
                 "description": "What to do with unknown milestones (`warn` by default).",
                 "enum": ["warn", "delete", "remove", "ignore", "skip"],
             },
+            # From https://docs.gitlab.com/ee/api/pipeline_schedules.html#create-a-new-pipeline-schedule
+            "pipeline_schedules": {
+                "type": "array",
+                "description": "The list of project's pipeline schedules",
+                "items": {
+                    "type": "object",
+                    "properties": {
+                        "description": {
+                            "type": "string",
+                            "description": "The description of pipeline schedule",
+                            "pattern": "[a-zA-Z0-9_]+",
+                        },
+                        "ref": {
+                            "type": "string",
+                            "description": "The branch/tag name will be triggered",
+                        },
+                        "cron": {
+                            "type": "string",
+                            "description": ("The cron (e.g. `0 1 * * *`) "
+                                            "([Cron syntax](https://en.wikipedia.org/wiki/Cron))"),
+                        },
+                        "cron_timezone ": {
+                            "type": "string",
+                            "description": ("The timezone supported by `ActiveSupport::TimeZone` "
+                                            "(e.g. `Pacific Time (US & Canada)`) (default: `'UTC'`)"),
+                        },
+                        "active": {
+                            "type": "boolean",
+                            "description": "The activation of pipeline schedule",
+                        },
+                        # From https://docs.gitlab.com/ee/api/pipeline_schedules.html
+                        # #create-a-new-pipeline-schedule-variable
+                        "variables": {
+                            "type": "array",
+                            "description": "The list of project's variables",
+                            "items": {
+                                "type": "object",
+                                "properties": {
+                                    "key": {
+                                        "type": "string",
+                                        "description": "The key of a variable",
+                                        "pattern": "[a-zA-Z0-9_]+",
+                                    },
+                                    "value": {
+                                        "type": "string",
+                                        "description": "The value of a variable",
+                                    },
+                                    "variable_type": {
+                                        "type": "string",
+                                        "description": ("The type of a variable. "
+                                                        "Available types are: env_var (default) and file"),
+                                        "enum": ["env_var", "file"],
+                                    },
+                                },
+                                "required": ["key", "value"],
+                                "additionalProperties": False,
+                            },
+                            "uniqueItems": True,
+                        },
+                        "unknown_variables": {  # GitLabracadabra
+                            "type": "string",
+                            "description": ("What to do with unknown pipeline schedule variables "
+                                            "(Value of `unknown_pipeline_schedule_variables` by default)."),
+                            "enum": ["warn", "delete", "remove", "ignore", "skip"],
+                        },
+                    },
+                    "required": ["description", "ref", "cron"],
+                    "additionalProperties": False,
+                },
+                "uniqueItems": True,
+                "_example": ("\n"
+                             "    - description: Build packages\n"
+                             "      ref: master\n"
+                             "      cron: '0 1 * * 5'\n"
+                             "      # cron_timezone: UTCs\n"
+                             "      # active: true\n"
+                             "      variables:\n"
+                             "        - key: MY_VAR\n"
+                             "          value: my value\n"
+                             "          # variable_type: env_var # or file\n"
+                             "      # unknown_variables: warn # one of warn, delete, remove, ignore, skip\n"),
+            },
+            "unknown_pipeline_schedules": {  # GitLabracadabra
+                "type": "string",
+                "description": "What to do with unknown pipeline schedules (`warn` by default).",
+                "enum": ["warn", "delete", "remove", "ignore", "skip"],
+            },
+            "unknown_pipeline_schedule_variables": {  # GitLabracadabra
+                "type": "string",
+                "description": "What to do with unknown pipeline schedule variables (`warn` by default).",
+                "enum": ["warn", "delete", "remove", "ignore", "skip"],
+            },
             # From https://docs.gitlab.com/ee/api/projects.html#edit-project
             "issues_enabled": {
                 "type": "boolean",
@@ -787,8 +884,8 @@ class GitLabracadabraProject(GitLabracadabraObject, LabelsMixin, MembersMixin, M
     }
 
     IGNORED_PARAMS = ['initialize_with_readme', 'unknown_groups', 'unknown_labels', 'unknown_members',
-                      'unknown_milestones', 'unknown_protected_branches', 'unknown_protected_tags', 'unknown_variables',
-                      'unknown_webhooks']
+                      'unknown_milestones', 'unknown_pipeline_schedules', 'unknown_pipeline_schedule_variables',
+                      'unknown_protected_branches', 'unknown_protected_tags', 'unknown_variables', 'unknown_webhooks']
 
     CREATE_KEY = 'name'
     CREATE_PARAMS = ['initialize_with_readme']
@@ -869,66 +966,6 @@ class GitLabracadabraProject(GitLabracadabraObject, LabelsMixin, MembersMixin, M
                 # Next branch will be created from this ref
                 ref = branch_name
 
-    """"_process_groups()
-
-    Process the groups param.
-    """
-    def _process_groups(self, param_name, param_value, dry_run=False, skip_save=False):
-        assert param_name == 'groups'  # noqa: S101
-        assert not skip_save  # noqa: S101
-        param_value = deepcopy(param_value)
-        unknown_groups = self._content.get('unknown_groups', 'warn')
-        # We first check for already shared groups
-        for group in self._obj.shared_with_groups:
-            if 'group_full_path' not in group:
-                # Gitlab < 11.8
-                # https://gitlab.com/gitlab-org/gitlab-ce/merge_requests/24052
-                group['group_full_path'] = GitLabracadabraGroup.get_full_path_from_id(group['group_id'])
-            GitLabracadabraGroup.map_group(group['group_id'], group['group_full_path'])
-            if group['group_full_path'] in param_value.keys():
-                target_access_level = access_level(param_value[group['group_full_path']])
-                if group['group_access_level'] != target_access_level:
-                    if dry_run:
-                        logger.info('[%s] NOT Changing group %s access level: %s -> %s (dry-run)',
-                                     self._name, group['group_full_path'],
-                                     group['group_access_level'], target_access_level)
-                    else:
-                        logger.info('[%s] Changing group %s access level: %s -> %s',
-                                     self._name, group['group_full_path'],
-                                     group['group_access_level'], target_access_level)
-                        self._obj.unshare(group['group_id'])
-                        self._obj.share(group['group_id'], target_access_level)
-                param_value.pop(group['group_full_path'])
-            else:
-                if unknown_groups in ['delete', 'remove']:
-                    if dry_run:
-                        logger.info('[%s] NOT Unsharing from unknown group: %s (dry-run)',
-                                    self._name, group['group_full_path'])
-                    else:
-                        logger.info('[%s] Unsharing from unknown group: %s',
-                                    self._name, group['group_full_path'])
-                        self._obj.unshare(group['group_id'])
-                elif unknown_groups not in ['ignore', 'skip']:
-                    logger.warning('[%s] NOT Unsharing from unknown group: %s (unknown_groups=%s)',
-                                   self._name, group['group_full_path'], unknown_groups)
-        # Remaining groups
-        for group_full_path, target_group_access in sorted(param_value.items()):
-            group_id = GitLabracadabraGroup.get_id_from_full_path(group_full_path)
-            if group_id is None:
-                logger.warning('[%s] Group not found %s',
-                                self._name, group_full_path)
-                continue
-            target_access_level = access_level(target_group_access)
-            if dry_run:
-                logger.info('[%s] NOT Sharing group %s: %s -> %s (dry-run)',
-                            self._name, group_full_path,
-                            0, target_access_level)
-            else:
-                logger.info('[%s] Sharing group %s: %s -> %s',
-                            self._name, group_full_path,
-                            0, target_access_level)
-                self._obj.share(group_id, target_access_level)
-
     """"_process_protected_branches()
 
     Process the protected_branches param.
