@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019 Mathieu Parent <math.parent@gmail.com>
+# Copyright (C) 2019-2020 Mathieu Parent <math.parent@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -178,7 +178,7 @@ class TestProject(TestCase):
             'groups': {'group_not_found': 'maintainer'},
             'unknown_groups': 'ignore',
         })
-        with patch('gitlabracadabra.objects.project.logger', autospec=True) as logger:
+        with patch('gitlabracadabra.mixins.groups.logger', autospec=True) as logger:
             obj.process()
             self.assertEqual(cass.all_played, True)
             logger.warning.assert_called_once_with('[%s] Group not found %s',
@@ -434,3 +434,58 @@ class TestProject(TestCase):
         })
         obj.process()
         self.assertEqual(cass.all_played, True)
+
+    @my_vcr.use_cassette
+    def test_pipeline_schedules(self, cass):
+        obj = GitLabracadabraProject('memory', 'test/test_pipeline_schedules', {
+            'pipeline_schedules': [
+                {
+                    "description": "create_me",
+                    "ref": "master",
+                    "cron": "0 1 * * 5",
+                    "cron_timezone": "UTC",
+                    "active": True,
+                },
+                {
+                    "description": "modify_me",
+                    "ref": "develop",
+                    "cron": "0 3 * * 5",
+                    "cron_timezone": "Pacific Time (US & Canada)",
+                    "active": False,
+                },
+            ],
+            'unknown_pipeline_schedules': 'delete',
+            'unknown_pipeline_schedule_variables': 'ignore',
+        })
+        obj.process()
+        self.assertEqual(cass.all_played, True)
+
+    @my_vcr.use_cassette
+    def test_pipeline_schedule_variables(self, cass):
+        obj = GitLabracadabraProject('memory', 'test/test_pipeline_schedule_variables', {
+            'pipeline_schedules': [
+                {
+                    "description": "some_schedule",
+                    "ref": "master",
+                    "cron": "0 16 * * *",
+                    "cron_timezone": "UTC",
+                    "active": False,
+                    "variables": [
+                        {
+                            "key": "create_me",
+                            "value": "some_value",
+                            "variable_type": "file",
+                        },
+                        {
+                            "key": "modify_me",
+                            "value": "new_value",
+                            "variable_type": "file",
+                        }
+                    ],
+                },
+            ],
+            'unknown_pipeline_schedules': 'delete',
+            'unknown_pipeline_schedule_variables': 'delete',
+        })
+        obj.process()
+        self.assertEqual(cass.all_played, True)
